/************************************************************************
 *   IRC - Internet Relay Chat, include/common.h
 *   Copyright (C) 1990 Armin Gruner
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __common_include__
#define __common_include__

#ifndef NEED_sys_types_h
# include <sys/types.h>
#endif

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef _WIN32
# define strcasecmp stricmp
# define strncasecmp strnicmp
#endif

#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef INADDR_NONE
# define INADDR_NONE ((in_addr_t) 0xffffffff)
#endif

#ifdef NEED_u_int32_t
typedef unsigned int u_int32_t;
#endif

#ifdef NEED_u_int
typedef unsigned int u_int;
#endif

#ifdef NEED_u_char
typedef unsigned char u_char;
#endif

#ifdef NEED_u_short
typedef unsigned short u_short;
#endif

#ifdef NEED_u_long
typedef unsigned long u_long;
#endif

#define CHANTYPES "#&+"
#define PREFIX "(ov)@+"
#define CHANMODES "beI,k,l,cfijmnprst"

#ifdef SOL20
#define INADDR_NONE (ipaddr_t)0xffffffffU
#endif

#ifndef PROTO
# if __STDC__
#  define PROTO(x) x
# else
#  define PROTO(x) ()
# endif
#endif

#ifndef NULL
# define NULL ((void*)0)
#endif

#ifdef TRUE
# undef TRUE
# define TRUE (1)
#endif

#ifdef FALSE
# undef FALSE
# define FALSE (0)
#endif

#ifndef UNSURE
# define UNSURE (2)
#endif

extern int match(char *, char *);
extern int ip_match(char *, char *);
extern char isstrdigit(char *);

#define mycmp(a,b) \
 ( (toupper((a)[0])!=toupper((b)[0])) || smycmp((a)+1,(b)+1) )
extern int smycmp (char *, char *);
extern int myncmp (char *, char *, int);

#ifdef NEED_strtok
extern char *strtok (char *, const char *);
#endif

#ifdef NEED_strtoken
extern char *strtoken (char **, char *, char *);
#endif

#ifdef NEED_inet_addr
extern unsigned long inet_addr (char *);
#endif

#ifdef NEED_strlcpy
size_t strlcpy(char *, const char *, size_t);
#endif

#ifdef NEED_strlcat
size_t strlcat(char *, const char *, size_t);
#endif

#ifdef NEED_daemon
int daemon(int, int);
#endif

#ifdef NEED_md5crypt
char *md5crypt(const char *, const char *);
#endif

#ifdef NEED_getopt
int getopt(int, char * const *, const char *);
#endif

#if defined(NEED_INET_NTOA) || defined(NEED_INET_NETOF) && !defined(_WIN32) \
	&& !defined(SCOUNIX)
#include <netinet/in.h>
#endif

#ifndef SCOUNIX /* SCO Openserver has these, but the header files are broken */
# ifdef NEED_INET_NTOA
extern char *inet_ntoa (struct in_addr);
# endif
#endif /* SCOUNIX */

#ifdef NEED_inet_netof
extern int inet_netof (struct in_addr);
#endif

int global_count;
extern char *myctime PROTO((time_t));
extern char *strtoken PROTO((char **, char *, char *));

#ifdef USE_CASETABLES
extern int casetable;
extern u_char *tolowertab, tolowertab1[], tolowertab2[];
extern u_char *touppertab, touppertab1[], touppertab2[];
#else
#ifndef SCOUNIX
extern u_char tolowertab[], touppertab[];
#else
extern unsigned char tolowertab[], touppertab[];
	/* Typedef for u_char exists and is wrong. */
#endif
#endif

#undef tolower
#define tolower(c) (tolowertab[(u_char)(c)])

#undef toupper
#define toupper(c) (touppertab[(u_char)(c)])

#undef isalpha
#undef isdigit
#undef isxdigit
#undef isalnum
#undef isprint
#undef isascii
#undef isgraph
#undef ispunct
#undef islower
#undef isupper
#undef isspace
#undef iscntrl

extern unsigned char char_atribs[];

#define PRINT 1
#define CNTRL 2
#define ALPHA 4
#define PUNCT 8
#define DIGIT 16
#define SPACE 32
#define ALLOW 64

#ifndef KLINE_TEMP
#define KLINE_PERM 0
#define KLINE_TEMP 1
#define KLINE_AKILL 2
#endif

#define iscntrl(c) (char_atribs[(u_char)(c)]&CNTRL)
#define isallowed(c) (char_atribs[(u_char)(c)]&ALLOW)
#define isalpha(c) (char_atribs[(u_char)(c)]&ALPHA)
#define isspace(c) (char_atribs[(u_char)(c)]&SPACE)
#define islower(c) ((char_atribs[(u_char)(c)]&ALPHA) && ((u_char)(c) > 0x5f))
#define isupper(c) ((char_atribs[(u_char)(c)]&ALPHA) && ((u_char)(c) < 0x60))
#define isdigit(c) (char_atribs[(u_char)(c)]&DIGIT)
#define isxdigit(c) (isdigit(c) || ('a' <= (c) && (c) <= 'f') || ('A' <= (c) && (c) <= 'F'))
#define isalnum(c) (char_atribs[(u_char)(c)]&(DIGIT|ALPHA))
#define isprint(c) (char_atribs[(u_char)(c)]&PRINT)
#define isascii(c) ((u_char)(c) >= 0 && (u_char)(c) <= 0x7f)
#define isgraph(c) ((char_atribs[(u_char)(c)]&PRINT) && ((u_char)(c) != 0x32))
#define ispunct(c) (!(char_atribs[(u_char)(c)]&(CNTRL|ALPHA|DIGIT)))

#define allow_forward(cptr, str) (!MyClient(cptr) || IsOper(cptr) || (strchr(str, '.') && !strchr(str, '?') && !strchr(str, '*')))

extern void flush_connections();
extern struct SLink *find_user_link(/* struct SLink *, struct Client * */);

#ifdef _WIN32
# undef strerror /* doesn't work with socket errors. */
#endif

extern int log_verbosity;

extern int lu_noninv, lu_inv, lu_serv, lu_oper,
	   lu_unknown, lu_channel, lu_lu, lu_lulocal, lu_lserv,
	   lu_clu, lu_mlu, lu_cglobalu, lu_mglobalu;

/* configuration options changed in runtime */
extern char *configfile;
extern char *network_name;
extern char *kline_address;
extern char *contact_url;
extern char *services_host;
extern char *server_mask;
extern char *ns_nick, *cs_nick, *ms_nick, *os_nick;
extern char *codepage_path;
extern char *opers_log, *clients_log;
extern int def_umode, nickchange_delay, show_connect_info, force_motd;

time_t now;

#endif /* __common_include__ */
