/* m_nick.c -- nickname registration / check;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 * $Id: m_nick.c,v 1.3 2002/11/18 12:44:08 hex Exp $
 */

#include "sys.h"
#include "struct.h"
#include "h.h"
#include "numeric.h"
#include "common.h"
#include "send.h"

extern int register_user(aClient *cptr, aClient *sptr,
	char *nick, char *username);

extern int m_user(aClient *, aClient *, int, char *[]);

int do_nick_name(char *nick)
{
	char *ch;

	if (*nick == '-' || isdigit(*nick))
		return 0;

	for (ch = nick; *ch && (ch - nick) < NICKLEN; ch++)
		if (!isvalid(*ch) || isspace(*ch))
			break;

	*ch = '\0';

	return (ch - nick);
}


/* returns zero if everything is fine and non-zero if a local connection
 * is trying to use a forbidden nickname.  
 */
static int check_qlines(aClient *cptr, aClient *sptr, char *nick)
{
	char *reason;
	aConfItem *aconf;
	aSqlineItem *asqline;

	if (IsServer(cptr))
		return 0;

	if ((aconf = find_conf_name(nick, CONF_QUARANTINED_NICK)) != NULL)
		reason = aconf->passwd;
	else if ((asqline = find_sqline_match(nick)) != NULL)
		reason = asqline->reason;
	else
		return 0;

	sendto_one(sptr, err_str(ERR_ERRONEUSNICKNAME), me.name,
		BadPtr(sptr->name) ? "*" : sptr->name,
		nick,
		BadPtr(reason) ? "reason unspecified" : reason);

	return 1;
}


static int delay_nick_change(aClient *acptr)
{
	time_t current;

	if (IsAnOper(acptr))
		return 0;

	time(&current);

	if (current < acptr->nextnick) {
		acptr->nextnick += 2;
		sendto_one(acptr, ":%s NOTICE %s :Nick change too fast, please "
			"wait %d seconds and try again.", me.name, acptr->name,
			acptr->nextnick - current);
		return 1;
	}

	acptr->nextnick += nickchange_delay;

	if (acptr->nextnick < current)
		acptr->nextnick = current;

	return 0;
}


/*
** m_nick
**     parv[0] = sender prefix
**     parv[1] = nickname
**  if from new client  -taz
**     parv[2] = nick password
**  if from server:
**      parv[2] = hopcount
**      parv[3] = timestamp
**      parv[4] = username
**      parv[5] = hostname
**      parv[6] = servername
**      parv[7] = servicestamp
**     parv[8] = info
*/
int m_nick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char nick[NICKLEN + 1];
	struct SLink *lp;
	time_t lastnick = (time_t) 0;
	int differ = 1;

	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NONICKNAMEGIVEN),
			me.name, parv[0]);
		return 0;
	}

	strlcpy(nick, parv[1], sizeof(nick));

	if (do_nick_name(nick) == 0 || (IsServer(cptr)
	&& strcmp(nick, parv[1]))) {
		sendto_one(sptr, err_str(ERR_ERRONEUSNICKNAME), me.name,
			parv[0], parv[1], "Illegal characters");

		if (IsServer(cptr)) {
			++ircstp->is_kill;

			sendto_ops("Bad Nick: %s From: %s %s", parv[1], parv[0],
				get_client_name(cptr, FALSE));

			if (!BadPtr(server_mask))
				sendto_one(cptr, ":%s KILL %s :Nick collision",
					me.name, parv[1]);
			else sendto_one(cptr,
					":%s KILL %s :%s (%s <- %s[%s])",
					me.name, parv[1], me.name, parv[1],
					nick, cptr->name);

			if (sptr != cptr) {
				if (!BadPtr(server_mask))
					sendto_serv_butone(cptr,
						":%s KILL %s :Nick collision",
						me.name, parv[0]);
				else sendto_serv_butone(cptr,
					":%s KILL %s :%s (%s <- %s!%s@%s)",
					me.name, parv[0], me.name,
					get_client_name(cptr, FALSE), parv[0],
					sptr->user ? sptr->username : "",
					sptr->user ? sptr->srvptr->name
					: cptr->name);
				sptr->flags |= FLAGS_KILLED;
				return exit_client(cptr, sptr, &me, "Bad nick");
			}
		}
		return 0;
	}

	if ((acptr = find_server(nick, NULL)) != NULL) {
		if (MyConnect(sptr)) {
			sendto_one(sptr, err_str(ERR_NICKNAMEINUSE), me.name,
				BadPtr(parv[0]) ? "*" : parv[0], nick);
			return 0;
		}

		++ircstp->is_kill;

		sendto_ops("Nick collision on %s(%s <- %s)", sptr->name,
			acptr->from->name, get_client_name(cptr, FALSE));
		sendto_one(cptr, ":%s KILL %s :%s (%s <- %s)", me.name,
			sptr->name, me.name, acptr->from->name,
			get_client_name(cptr, FALSE));
		sptr->flags |= FLAGS_KILLED;
		return exit_client(cptr, sptr, &me, "Nick/Server collision");
	}

	if (check_qlines(cptr, sptr, nick)) {
		sendto_realops("Q-lined nick %s from %s.", nick,
			get_client_name(cptr, FALSE));
		return 0;
	}

	if (MyClient(cptr) && !IsAnOper(cptr))
		cptr->since += 3;

	if ((acptr = find_client(nick, NULL)) == NULL)
		goto nickkilldone;

	if (IsUnknown(acptr) && MyConnect(acptr)) {
		if (acptr == cptr)
			return 0;
		acptr->flags |= FLAGS_KILLED;
		exit_client(NULL, acptr, &me, "Overridden");
		goto nickkilldone;
	}

	if (acptr->user == NULL) {
		++ircstp->is_kill;
		sendto_ops("Lost user field for %s in change from %s",
			acptr->name, get_client_name(cptr, FALSE));
		sendto_one(acptr, ":%s KILL %s :%s (Lost user field!)",
			me.name, acptr->name, me.name);
		acptr->flags |= FLAGS_KILLED;
		exit_client(NULL, acptr, &me, "Lost user field");
		goto nickkilldone;
	}

	if (acptr == sptr) {
		if (strcmp(acptr->name, nick) == 0)
			return 0;
		goto nickkilldone;
	}

	if (!IsServer(cptr)) {
		sendto_one(sptr, err_str(ERR_NICKNAMEINUSE), me.name,
			BadPtr(parv[0]) ? "*" : parv[0], nick);
		return 0;
	}

	if (IsServer(sptr)) {
#ifdef USE_CASETABLES
		if ((find_client(parv[0], NULL) != acptr)
		&& !IsServer(find_client(parv[0], NULL))) {
			sendto_one(sptr, ":%s KILL %s :%s (%s Self collision)",
				me.name, nick, me.name,
				get_client_name(cptr, FALSE));
			return 0;
		}
#endif

		if (parc > 3) {
			lastnick = atoi(parv[3]);
			if (parc > 5)
				differ = (mycmp(acptr->user->username, parv[4])
					|| mycmp(acptr->user->host, parv[5]));
		}

		sendto_failops("Nick collision on %s (%s %d <- %s %d)",
			acptr->name, acptr->from->name, acptr->lastnick,
			get_client_name(cptr, FALSE), lastnick);
	}
	else {
		if (parc > 2)
			lastnick = atoi(parv[2]);
		differ = (mycmp(acptr->user->username, sptr->user->username)
			|| mycmp(acptr->user->host, sptr->user->host));

		sendto_locfailops("Nick change collision from %s to %s "
			"(%s %d <- %s %d)", sptr->name, acptr->name,
			acptr->from->name, acptr->lastnick,
			get_client_name(cptr, FALSE), lastnick);
	}

	if (acptr->from != cptr) {
		if (!lastnick || (differ && lastnick >= acptr->lastnick)
		|| (!differ && lastnick <= acptr->lastnick)) {
			if (!IsServer(sptr)) {
				++ircstp->is_kill;
				sendto_serv_butone(cptr,
					":%s KILL %s :%s (%s <- %s)",
					me.name, sptr->name, me.name,
					acptr->from->name,
					get_client_name(cptr, FALSE));
				sptr->flags |= FLAGS_KILLED;
				exit_client(NULL, sptr, &me,
					"Nick collision (you're a ghost)");
			}
			if (lastnick && lastnick != acptr->lastnick)
				return 0;
		}
		sendto_one(acptr, err_str(ERR_NICKCOLLISION),
			me.name, acptr->name, nick);
	}

	++ircstp->is_kill;
	sendto_serv_butone(cptr, ":%s KILL %s :%s (%s <- %s)", me.name,
		acptr->name, me.name, acptr->from->name,
		get_client_name(cptr, FALSE));
	acptr->flags |= FLAGS_KILLED;
	exit_client(NULL, acptr, &me, "Nick collision (older nick overruled)");

	if (lastnick == acptr->lastnick)
		return 0;

nickkilldone:
	if (IsServer(sptr)) {
		sptr = make_client(cptr, find_server(parv[6], NULL));
		add_client_to_list(sptr);

		if (parc > 2)
			sptr->hopcount = atoi(parv[2]);
		if (parc > 3)
			sptr->lastnick = atoi(parv[3]);
		else
			sptr->lastnick = time(NULL);
	}
	else if (sptr->name[0] && IsPerson(sptr)) {
		if (MyClient(sptr)) {
			for (lp = cptr->user->channel; lp; lp = lp->next) {
				if (is_banned(cptr, lp->value.chptr, 1)) {
					sendto_one(cptr,
						err_str(ERR_BANNICKCHANGE),
						me.name, parv[0],
						lp->value.chptr->chname);
					return 0;
				}
			}

			if (delay_nick_change(sptr))
				return 0;
		}

		if (sptr->umodes & UMODE_REGNICK) {
			sptr->umodes &= ~UMODE_REGNICK;
			if (MyClient(sptr))
				sendto_one(sptr, ":%s MODE %s :-r", sptr->name,
					sptr->name);
		}

		if (strcasecmp(parv[0], nick))
			sptr->lastnick = (MyClient(sptr) || parc < 3)
				? time(NULL) : atoi(parv[2]);

		add_history(sptr);
		sendto_common_channels(sptr, ":%s NICK :%s", parv[0], nick);
		sendto_serv_butone(cptr, ":%s NICK %s :%d", parv[0],
			nick, sptr->lastnick);
	}
	else if (sptr->name[0] == '\0') {
		if (!BadPtr(contact_url))
			sendto_one(sptr, ":%s NOTICE %s :*** If you need "
				"assistance with a connection problem, please "
				"refer to %s", me.name, nick, contact_url);

		if ((parc > 2) && (strlen(parv[2]) < sizeof(sptr->passwd)))
			strcpy(sptr->passwd, parv[2]);

		strcpy(sptr->name, nick);
		if (sptr->user) {
			sptr->lastnick = time(NULL);

			if (register_user(cptr, sptr, nick,
			sptr->user->username) == FLUSH_BUFFER)
				return FLUSH_BUFFER;
		}
	}

	if (sptr->name[0] != '\0') {
		del_from_client_hash_table(sptr->name, sptr);
		if (IsPerson(sptr))
			hash_check_notify(sptr, RPL_LOGOFF);
	}

	strcpy(sptr->name, nick);
	add_to_client_hash_table(nick, sptr);

	if (IsServer(cptr) && parc > 7) {
		parv[3] = nick;
		return m_user(cptr, sptr, parc - 3, &parv[3]);
	}
	else if (IsPerson(sptr))
		hash_check_notify(sptr, RPL_LOGON);

	return 0;
}
