/* m_pong.c -- reply to a PING command, used for link delay measurement;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 * $Id: m_pong.c,v 1.10 2002/11/18 12:44:08 hex Exp $
 */

#include "struct.h"
#include "send.h"
#include "numeric.h"
#include "h.h"

/* m_pong
 *	parv[0] = sender prefix (server)
 *	parv[1] = origin (client)
 *	parv[2] = source (server)
 *	parv[3] = timestamp (original)
 *	parv[4] = ballast (indicating that it's travelling to the originator)
 */

int m_pong(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient dummy;
	char timebuf[16];

	if (parc < 2 || BadPtr(parv[1])) {
		sendto_one(sptr, err_str(ERR_NOORIGIN), me.name, parv[0]);
		return 0;
	}

	cptr->flags &= ~FLAGS_PINGSENT;
	sptr->flags &= ~FLAGS_PINGSENT;

	if (MyClient(sptr))
		return 0;

	/* travelling to the source */
	if (parc == 4) {
		if (hunt_server(cptr, sptr, ":%s PONG %s %s %s", 2, parc, parv)
			!= HUNTED_ISME) return 0;

		/* I'm the source, adjust the delay and add the ballast */
		sprintf(timebuf, "%u", get_ticks() - (unsigned int)atoi(parv[3]));
		parv[3] = timebuf;
		parv[4] = timebuf; /* ballast */
		parc = 5;

		/* the direction might have changed, reset the path */
		memset(&dummy, 0, sizeof(dummy));
		cptr = sptr = &dummy;
	}

	/* travelling to the originator */
	if (parc == 5) {
		aClient *acptr;

		if (hunt_server(cptr, sptr, ":%s PONG %s %s %s %s", 1, parc,
			parv) != HUNTED_ISME) return 0;

		/* the original sender is ours, check if it still exists */
		if (!(acptr = find_client(parv[1], NULL)))
			return 0;

		sendto_one(acptr, ":%s PONG %s :%s ms delay with %s",
			parv[2], acptr->name, parv[3], parv[0]);
	}

	return 0;
}
