/* m_watch.c -- server side notify;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 * $Id: m_watch.c,v 1.4 2002/11/18 12:44:08 hex Exp $
 */

#include <string.h>

#include "sys.h"
#include "struct.h"
#include "h.h"
#include "numeric.h"
#include "send.h"

/*
 * RPL_NOWON - Online at the moment (Succesfully added to WATCH-list)
 * RPL_NOWOFF - Offline at the moement (Succesfully added to WATCH-list)
 * RPL_WATCHOFF - Succesfully removed from WATCH-list.
 * ERR_TOOMANYWATCH - Too many WATCH entries.
 */

static void show_watch(aClient * cptr, char *name, int rpl1, int rpl2)
{
	aClient *acptr;

	if ((acptr = find_person(name, NULL)))
		sendto_one(cptr, rpl_str(rpl1), me.name, cptr->name,
			acptr->name, acptr->user->username,
			IsHMask(acptr) ? acptr->user->vhost : acptr->user->host,
			acptr->lastnick);
	else
		sendto_one(cptr, rpl_str(rpl2), me.name, cptr->name,
			name, "*", "*", 0);
}


int m_watch(aClient * cptr, aClient * sptr, int parc, char *parv[])
{
	char buf[BUFSIZE], *s, **pav = parv, *user,
		*p = NULL, *def = "l";
	aClient *acptr;

	if (check_registered(sptr))
		return 0;

	if (parc < 2) {
		parc = 2;
		parv[1] = def;
	}

	for (s = (char *) strtoken(&p, *++pav, " "); s;
	s = (char *) strtoken(&p, NULL, " ")) {
		if ((user = strchr(s, '!')) != NULL)
			*user++ = '\0';

		if (*s == '+') {
			if (do_nick_name(s + 1)) {
				if (sptr->notifies >= MAXWATCH) {
					sendto_one(sptr,
						err_str(ERR_TOOMANYWATCH),
						me.name, cptr->name, s + 1);
					continue;
				}
				add_to_notify_hash_table(s + 1, sptr);
			}

			show_watch(sptr, s + 1, RPL_NOWON, RPL_NOWOFF);
			continue;
		}

		if (*s == '-') {
			del_from_notify_hash_table(s + 1, sptr);
			show_watch(sptr, s + 1, RPL_WATCHOFF, RPL_WATCHOFF);
			continue;
		}

		if (*s == 'C' || *s == 'c') {
			hash_del_notify_list(sptr);
			continue;
		}

		if (*s == 'S' || *s == 's') {
			Link *lp;
			aNotify *anptr;
			int count = 0;

			/* Send a list of how many users they have on their
			 * WATCH list and how many WATCH lists they are on.
			 */
			anptr = hash_get_notify(sptr->name);
			if (anptr)
				for (lp = anptr->notify, count = 1;
					(lp = lp->next); count++);

			sendto_one(sptr, rpl_str(RPL_WATCHSTAT), me.name,
				parv[0], sptr->notifies, count);

			if ((lp = sptr->notify) == NULL) {
				sendto_one(sptr, rpl_str(RPL_ENDOFWATCHLIST),
					me.name, parv[0], *s);
				continue;
			}

			*buf = '\0';
			strcpy(buf, lp->value.nptr->nick);
			count = strlen(parv[0]) + strlen(me.name)
				+ 10 + strlen(buf);

			while ((lp = lp->next)) {
				if (count + strlen(lp->value.nptr->nick)
				+ 1 > BUFSIZE - 2) {
					sendto_one(sptr, rpl_str(RPL_WATCHLIST),
						me.name, parv[0], buf);
					*buf = '\0';
					count = strlen(parv[0])
						+ strlen(me.name) + 10;
				}
				strcat(buf, " ");
				strcat(buf, lp->value.nptr->nick);
				count += (strlen(lp->value.nptr->nick) + 1);
			}
			sendto_one(sptr, rpl_str(RPL_WATCHLIST), me.name,
				parv[0], buf);
			sendto_one(sptr, rpl_str(RPL_ENDOFWATCHLIST), me.name,
				parv[0], *s);
			continue;
		}

		if (*s == 'L' || *s == 'l') {
			Link *lp = sptr->notify;

			while (lp) {
				if ((acptr = find_person(lp->value.nptr->nick,
				NULL)))
					sendto_one(sptr, rpl_str(RPL_NOWON),
						me.name, parv[0], acptr->name,
						acptr->user->username,
						acptr->user->host,
						acptr->lastnick);
				else if (isupper(*s))
					sendto_one(sptr, rpl_str(RPL_NOWOFF),
						me.name, parv[0],
						lp->value.nptr->nick, "*", "*",
						lp->value.nptr->lasttime);
				lp = lp->next;
			}

			sendto_one(sptr, rpl_str(RPL_ENDOFWATCHLIST), me.name,
				parv[0], *s);
			continue;
		}
	}

	return 0;
}
