/* m_whois.c -- get detailed information about matching nicks;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 * $Id: m_whois.c,v 1.23 2002/11/18 12:44:08 hex Exp $
 */

#include <time.h>

#include "config.h"
#include "struct.h"
#include "send.h"
#include "numeric.h"
#include "h.h"
#include "channel.h"

__inline static char* get_host_mask(aClient *acptr)
{
	return IsHMask(acptr) ? acptr->user->vhost : acptr->user->host;
}


/* parv[0] = sender prefix, parv[1] = nickname masklist */
int m_whois(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static anUser UnknownUser = {
		NULL, /* nextu */
		NULL, /* channel */
		NULL, /* invited */
		NULL, /* silence */
		0, /* last */
		0, /* servicestamp */
		1, /* refcount */
		0, /* joined */
		"<Unknown>", /* username */
		"<Unknown>", /* host */
		"<Unknown>" /* server */
	};
	Link *lp;
	anUser *user;
	aClient *acptr, *a2cptr;
	aChannel *chptr;
	char *nick, *tmp, *name, buf[1024], *p = (char*)NULL;
	int found, len, mlen;

	if (check_registered_user(sptr))
		return 0;

	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NONICKNAMEGIVEN),
			me.name, parv[0]);
		return 0;
	}

	if (MyClient(sptr) && (parc > 2) && strcasecmp(parv[1], parv[2])
	&& !IsOper(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc > 2) {
		if (hunt_server(cptr, sptr, ":%s WHOIS %s :%s", 1, parc, parv)
			!= HUNTED_ISME) return 0;
		parv[1] = parv[2];
	}

	for (tmp = parv[1]; (nick = strtoken(&p, tmp, ",")); tmp = NULL) {
		int invis, showperson, member, wilds;

		found = 0;
		collapse(nick);
		wilds = (strchr(nick, '?') || strchr(nick, '*'));
		if (wilds && IsServer(cptr))
			continue;
		for (acptr = client; (acptr = next_client(acptr, nick)); acptr = acptr->next) {
			int su = 0;

			if (!IsClient(acptr) /* IsServer(acptr) */)
				continue;

			if (IsMe(acptr))
				break;

			if (BadPtr(server_mask) || IsAnOper(sptr) || sptr == acptr)
				su = 1;

			/* 'Rules' established for sending a WHOIS reply:
			 *
			 * - only allow a remote client to get replies for
			 *   local clients if wildcards are being used;
			 *
			 * - if wildcards are being used dont send a reply if
			 *   the querier isnt any common channels and the
			 *   client in question is invisible and wildcards are
			 *   in use (allow exact matches only);
			 *
			 * - only send replies about common or public channels
			 *   the target user(s) are on;
			 */

			if (!MyConnect(sptr) && !MyConnect(acptr) && wilds)
				continue;

			user = acptr->user ? acptr->user : &UnknownUser;
			name = (!*acptr->name) ? "?" : acptr->name;

			invis = acptr!=sptr && IsInvisible(acptr);
			member = (user->channel) ? 1 : 0;
			showperson = (wilds && !invis && !member) || !wilds;
			for (lp = user->channel; lp; lp = lp->next) {
				chptr = lp->value.chptr;
				member = IsMember(sptr, chptr);

				if (invis && !member)
					continue;

				if (member || (!invis && !SecretChannel(chptr))) {
					showperson = 1;
					break;
				}
			}
			if (!showperson)
				continue;

			a2cptr = acptr->srvptr;

			sendto_one(sptr, rpl_str(RPL_WHOISUSER), me.name, parv[0], name, user->username, get_host_mask(acptr), acptr->info);

			if (IsHMask(acptr) && (IsOper(sptr) || (acptr == sptr) || (IsLocOp(sptr) && sptr->srvptr == acptr->srvptr)))
				sendto_one(sptr,rpl_str(RPL_WHOISHOST), me.name, parv[0], name, user->host);

			if (IsARegNick(acptr))
				sendto_one(sptr, rpl_str(RPL_WHOISREGNICK), me.name, parv[0], name);

			found = 1;
			mlen = strlen(me.name) + strlen(parv[0]) + 6 + strlen(name);

			if (!IsULine(acptr, acptr)) {
				for (len = 0, *buf = '\0', lp = user->channel; lp; lp = lp->next) {
					chptr = lp->value.chptr;
					if (!SecretChannel(chptr) || IsMember(sptr, chptr) || (sptr == acptr)) {
						if (len + strlen(chptr->chname) > (size_t) BUFSIZE - 4 - mlen) {
							sendto_one(sptr, ":%s %d %s %s :%s", me.name, RPL_WHOISCHANNELS, parv[0], name, buf);
							*buf = '\0';
							len = 0;
						}
						if (has_op(acptr, chptr))
							*(buf + len++) = '@';
						else if (has_voice(acptr, chptr))
							*(buf + len++) = '+';
						if (len)
							*(buf + len) = '\0';
						(void)strcpy(buf + len, chptr->chname);
						len += strlen(chptr->chname);
						(void)strcat(buf + len, " ");
						len++;
					}
				}

				if (buf[0] != '\0')
					sendto_one(sptr, rpl_str(RPL_WHOISCHANNELS), me.name, parv[0], name, buf);
			}

			sendto_one(sptr, rpl_str(RPL_WHOISSERVER),
				me.name, parv[0], name,
				(su || BadPtr(server_mask)) ? a2cptr->name : server_mask,
				su ? a2cptr->info : "Wonderland...");

			if (IsAway(acptr))
				sendto_one(sptr, rpl_str(RPL_AWAY), me.name,
					parv[0], name,
					(MyClient(acptr) && (su || (parc > 2))) ? acptr->away : "Gone.");

			buf[0]='\0';
			if (IsSAdmin(acptr))
				strcpy(buf, IsAnOper(acptr) ? "an IRC Operator - Services Administrator" : "a Services Administrator");
			else if (IsAnOper(sptr) && IsLocOp(acptr))
				strcpy(buf, "a local IRC Operator");
			else if (IsAnOper(acptr))
				strcpy(buf, "an IRC Operator");

			if (buf[0])
				sendto_one(sptr, rpl_str(RPL_WHOISOPERATOR),
					me.name, parv[0], name, buf);

			if (MyConnect(acptr) && (su || (parc > 2)))
				sendto_one(sptr, rpl_str(RPL_WHOISIDLE),
					me.name, parv[0], name,
					time(NULL) - user->last,
					acptr->firsttime);

			if ((acptr->umodes & UMODE_UWNOTICE) && (acptr != sptr)) {
				sendto_one(acptr, ":%s NOTICE %s :WHOIS from %s!%s@%s",
					me.name, acptr->name, sptr->name,
					sptr->user->username, get_host_mask(sptr));
				sendto_syschan("debug", "whois notice from %s to %s",
					sptr->name, acptr->name);
			}
		}
		if (!found)
			sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, parv[0], nick);
		if (p)
			p[-1] = ',';
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFWHOIS), me.name, parv[0], parv[1]);
	return 0;
}
