/* s_conf.c -- configuration file and lines processing;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

/* Changed all calls of check_pings so that only when a kline-related command
   is used will a kline check occur -- Barubary */

#define KLINE_RET_AKILL 3
#define KLINE_RET_PERM 2
#define KLINE_RET_DELOK 1
#define KLINE_DEL_ERR 0

#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#ifndef _WIN32
# include <sys/socket.h>
# include <sys/wait.h>
# include <unistd.h>
# include <netinet/in.h>
# include <arpa/inet.h>
#else
# include <io.h>
#endif
#include <sys/stat.h>
#ifdef __hpux
# include "inet.h"
#endif
# include <time.h>
#ifdef R_LINES
# include <signal.h>
#endif

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "channel.h"
#include "send.h"
#include "h.h"

static int check_time_interval PROTO((char *, char *));
static int lookup_confhost PROTO((struct ConfItem *));
static int is_comment PROTO((char *));

extern void delist_conf(aConfItem *);
extern void free_sqline(aSqlineItem *);

aSqlineItem *sqline = NULL;
struct ConfItem *conf = NULL;
char gcomment[256];

char check_ipmask(char *mask)
{
	char str[32], *p, *state;
	u_long bf;

	if (!mask || !*mask)
		return 1;

	if (strlen(mask) > 20)
		return 1;
	if ((p = strchr(mask, '/'))) {
		strncpyzt(str, mask,(u_long)p - (u_long)mask + 1);
		p++;
		bf = atoi(p);
		if (bf > 32)
			return 1;
		p = strtoken(&state, str, ".");
		if (p == NULL || !isstrdigit(p))
			return 1;
		p = strtoken(&state, 0, ".");
		if (p == NULL || !isstrdigit(p))
			return 1;
		p = strtoken(&state, 0, ".");
		if (p == NULL || !isstrdigit(p))
			return 1;
		p = strtoken(&state, 0, ".");
		if (p == NULL || !isstrdigit(p))
			return 1;
		p = strtoken(&state, 0, ".");
		if (p != NULL)
			return 1;
		return 0;
	}
	strcpy(str, mask);
	p = strtoken(&state, str, ".");
	if (p == NULL)
		return 1;
	p = strtoken(&state, 0, ".");
	if (p == NULL)
		return 1;
	p = strtoken(&state, 0, ".");
	if (p == NULL)
		return 1;
	p = strtoken(&state, 0, ".");
	if (p == NULL)
		return 1;
	p = strtoken(&state, 0, ".");
	if (p != NULL)
		return 1;
	return 0;
}

static int check_hostname(char *host)
{
	if (!host || !*host || strlen(host) > HOSTLEN)
		return 1;

	while (*host) {
		if (*host == '.') {
			host++;

			if (*host == '.')
				return 1;
		}

		if (*host == '-') {
			host++;
			continue;
		}

		if (*host > 96 && *host < 123) {
			host++;
			continue;
		}

		if (*host > 64 && *host < 91) {
			host++;
			continue;
		}

		if (*host > 47 && *host < 58) {
			host++;
			continue;
		}
		return 1;
	}
	return 0;
}


/* remove all conf entries from the client except those which match
 * the status field mask.
 */
void det_confs_butmask(aClient *cptr, int mask)
{
	Link *tmp, *tmp2;

	for (tmp = cptr->confs; tmp; tmp = tmp2) {
		tmp2 = tmp->next;
		if ((tmp->value.aconf->status & mask) == 0)
			detach_conf(cptr, tmp->value.aconf);
	}
}


/* Add a temporary line to the configuration
 * temp: 0 = perm 1 = temp 2 = akill
 */
void add_temp_conf(unsigned int status, char *host, char *passwd, char *name, int port, int class, int temp)
{
	struct ConfItem *aconf;

	aconf = make_conf();

	aconf->tmpconf = temp;
	aconf->status = status;

	if (host)
		aconf->host = strdup(host);

	if (passwd)
		aconf->passwd = strdup(passwd);

	if (name)
		aconf->name = strdup(name);

	aconf->port = port;

	if (class)
		Class(aconf) = find_class(class);

	if (!find_temp_conf_entry(aconf, status)) {
		aconf->next = conf;
		conf = aconf;
		aconf = NULL;
	}

	if (aconf)
		free_conf(aconf);
}

/* delete a temporary conf line.  *only* temporary conf lines may be deleted. */
int del_temp_conf(unsigned int status, char *host, char *passwd, char *name, int port, int class, unsigned int akill)
{
	struct ConfItem *aconf;
	struct ConfItem *bconf;
	u_int mask;
	u_int result=KLINE_DEL_ERR;

	aconf = make_conf();

	aconf->status=status;

	if(host)
		aconf->host = strdup(host);

	if(passwd)
		aconf->passwd = strdup(passwd);

	if(name)
		aconf->name = strdup(name);

	aconf->port = port;

	if(class)
		Class(aconf) = find_class(class);

	mask = status;
	if ((bconf = find_temp_conf_entry(aconf,mask)) != NULL) {
		/* Completely skirt the akill error messages if akill is set to 1
		 * this allows RAKILL to do its thing without having to go through the
		 * error checkers.  If it had to it would go kaplooey. --Russell
		 */
		if (bconf->tmpconf == KLINE_PERM && (akill != 3))
			result = KLINE_RET_PERM; /* Kline permanent */
		else if (!akill && (bconf->tmpconf == KLINE_AKILL))
			result = KLINE_RET_AKILL; /* Akill */
		else if (akill && (bconf->tmpconf != KLINE_AKILL))
			result = KLINE_RET_PERM;
		else {
			bconf->status |= CONF_ILLEGAL; /* just mark illegal */
			result = KLINE_RET_DELOK; /* same as deletion */
		}
	}

	if (aconf)
		free_conf(aconf);

	return result; /* if it gets to here, it doesn't exist */
}

/* find the first (best) I line to attach. */
int attach_Iline(aClient *cptr, struct hostent *hp, char *sockhost)
{
	int i;
	char *hname;
	struct ConfItem *aconf;
	static char uhost[HOSTLEN+USERLEN+3], fullname[HOSTLEN+1];

	for (aconf = conf; aconf; aconf = aconf->next) {
		if (aconf->status != CONF_CLIENT)
			continue;
		if (aconf->port && aconf->port != cptr->acpt->port)
			continue;
		if (!aconf->host || !aconf->name)
			goto attach_iline;
		if (hp)
			for (i = 0, hname = hp->h_name; hname; hname = hp->h_aliases[i++]) {
				strncpy(fullname, hname, sizeof(fullname)-1);
				add_local_domain(fullname, HOSTLEN - strlen(fullname));
				Debug((DEBUG_DNS, "a_il: %s->%s", sockhost, fullname));
				if (strchr(aconf->name, '@')) {
					strcpy(uhost, cptr->username);
					strcat(uhost, "@");
				}
				else
					*uhost = '\0';
				strncat(uhost, fullname, sizeof(uhost) - strlen(uhost));
				if (!match(aconf->name, uhost))
					goto attach_iline;
			}

		if (strchr(aconf->host, '@')) {
			strncpyzt(uhost, cptr->username, sizeof(uhost));
			strcat(uhost, "@");
		}
		else
			*uhost = '\0';

		strncat(uhost, sockhost, sizeof(uhost) - strlen(uhost));
		if (!ip_match(aconf->host,uhost) || !match(aconf->host,uhost))
			goto attach_iline;

		if (*cptr->vhost) {
			if (strchr(aconf->name, '@')) {
				strncpyzt(uhost, cptr->username, sizeof(uhost));
				strcat(uhost, "@");
			}
			else
				*uhost = '\0';
			strncat(uhost, cptr->vhost, sizeof(uhost) - strlen(uhost));
			if (!match(aconf->name, uhost))
				goto attach_iline;
		}

		continue;

attach_iline:
		if (strchr(uhost, '@'))
			cptr->flags |= FLAGS_DOID;
		get_sockhost(cptr, uhost);
		return attach_conf(cptr, aconf);
	}
	return -1;
}

/* Find the single N line and return pointer to it (from list).
 * If more than one then return NULL pointer.
 */
struct ConfItem *count_cnlines(Link *lp)
{
	struct ConfItem *aconf, *cline = NULL, *nline = NULL;

	for (; lp; lp = lp->next) {
		aconf = lp->value.aconf;
		if (!(aconf->status & CONF_SERVER_MASK))
			continue;
		if (aconf->status == CONF_CONNECT_SERVER && !cline)
			cline = aconf;
		else if (aconf->status == CONF_NOCONNECT_SERVER && !nline)
			nline = aconf;
	}
	return nline;
}

/* detach_conf
 *     Disassociate configuration from the client.
 *      Also removes a class from the list if marked for deleting.
 */
int detach_conf(aClient *cptr, struct ConfItem *aconf)
{
	Link **lp, *tmp;

	lp = &(cptr->confs);

	while (*lp) {
		if ((*lp)->value.aconf == aconf) {
			if ((aconf) && (Class(aconf))) {
				if (aconf->status & CONF_CLIENT_MASK)
					if (ConfLinks(aconf) > 0)
						--ConfLinks(aconf);

				if (ConfMaxLinks(aconf) == -1 && ConfLinks(aconf) == 0) {
					free_class(Class(aconf));
					Class(aconf) = NULL;
				}
			}

			if (aconf && !--aconf->clients && IsIllegal(aconf))
				free_conf(aconf);

			tmp = *lp;
			*lp = tmp->next;
			free_link(tmp);
			return 0;
		}
		else
			lp = &((*lp)->next);
	}
	return -1;
}

static int is_attached(struct ConfItem *aconf, aClient *cptr)
{
	Link *lp;

	for (lp = cptr->confs; lp; lp = lp->next)
		if (lp->value.aconf == aconf)
			break;

	return (lp) ? 1 : 0;
}

/* Borg88
Calculate number of clients with same IP and using same Conf Line
1 - too many clones
0 - okey
*/

static char TooManyClones(aClient *cptr, struct ConfItem *conf)
{
	int i, cnt;
	Link *lp;
	aClient *acptr;
	struct ConfItem *aconf;

	if(ConfConFreq(conf)==0)
		return(0);

	cnt=0;
	for (i = 0; i <= highest_fd; i++) {
		if ((acptr=local[i]) && acptr->ip.s_addr==cptr->ip.s_addr) {
			for(lp = acptr->confs; lp; lp = lp->next) {
				aconf = lp->value.aconf;
				if (aconf->status & (CONF_LOCOP|CONF_OPERATOR|CONF_CLIENT) && ConfClass(aconf) == ConfClass(conf))
					break;
			}
			if (lp)
				cnt++;
		}
	}

	return (cnt >= ConfConFreq(conf)) ? 1 : 0;
}


/* attach_conf
 * Associate a specific configuration entry to a *local*
 * client (this is the one which used in accepting the
 * connection). Note, that this automaticly changes the
 * attachment if there was an old one...
 */
int attach_conf(aClient *cptr, struct ConfItem *aconf)
{
	Link *lp;

	if (is_attached(aconf, cptr) || IsIllegal(aconf))
		return 1;

	if ((aconf->status & (CONF_LOCOP | CONF_OPERATOR | CONF_CLIENT)) && aconf->clients >= ConfMaxLinks(aconf) && ConfMaxLinks(aconf) > 0)
		return -3; /* Use this for printing error message */

	if ((aconf->status & CONF_CLIENT) && TooManyClones(cptr, aconf))
		return(-4);

	lp = make_link();
	lp->next = cptr->confs;
	lp->value.aconf = aconf;
	cptr->confs = lp;
	aconf->clients++;

	if (aconf->status & CONF_CLIENT_MASK)
		ConfLinks(aconf)++;

	return 0;
}


struct ConfItem *find_admin()
{
	struct ConfItem *aconf;

	for (aconf = conf; aconf; aconf = aconf->next)
		if (aconf->status & CONF_ADMIN)
			break;

	return (aconf);
}


/* Find a DR_PASS line for the /DIE or /RESTART command
 * Instead of returning the whole structure we return a
 * char* which is the pass.
 * Added December 28 1997 -- NikB
 */
char *find_diepass()
{
	struct ConfItem *aconf;

	for (aconf = conf; aconf; aconf = aconf->next)
		if (aconf->status & CONF_DRPASS)
			return (aconf->host);

	return NULL; /* Return NULL (We did not find any) */
}


char *find_restartpass()
{
	struct ConfItem *aconf;

	for (aconf = conf; aconf; aconf = aconf->next)
		if (aconf->status & CONF_DRPASS)
			return (aconf->passwd);

	return NULL; /* Return NULL (We did not find any) */
}


struct ConfItem *find_me()
{
	struct ConfItem *aconf;
	for (aconf = conf; aconf; aconf = aconf->next)
		if (aconf->status & CONF_ME)
			break;

	return (aconf);
}

/* attach_confs
 *  Attach a CONF line to a client if the name passed matches that for
 * the conf file (for non-C/N lines) or is an exact match (C/N lines
 * only).  The difference in behaviour is to stop C:*::* and N:*::*.
 */
struct ConfItem *attach_confs(aClient *cptr, char *name, int statmask)
{
	struct ConfItem *tmp, *first = NULL;
	int len = strlen(name);

	if (!name || len > HOSTLEN)
		return NULL;

	for (tmp = conf; tmp; tmp = tmp->next) {
		if ((tmp->status & statmask) && !IsIllegal(tmp) && ((tmp->status & (CONF_SERVER_MASK|CONF_HUB)) == 0) && tmp->name && !match(tmp->name, name)) {
			if (!attach_conf(cptr, tmp) && !first)
				first = tmp;
		}
		else if ((tmp->status & statmask) && !IsIllegal(tmp) && (tmp->status & (CONF_SERVER_MASK|CONF_HUB)) && tmp->name && !mycmp(tmp->name, name)) {
			if (!attach_conf(cptr, tmp) && !first)
			first = tmp;
		}
	}
	return (first);
}

/* Added for new access check    meLazy */
struct ConfItem *attach_confs_host(aClient *cptr, char *host, int statmask)
{
	struct ConfItem *tmp, *first = NULL;
	int len = strlen(host);

	if (!host || len > HOSTLEN)
		return NULL;

	for (tmp = conf; tmp; tmp = tmp->next) {
		if ((tmp->status & statmask) && !IsIllegal(tmp) && (tmp->status & CONF_SERVER_MASK) == 0 && (!tmp->host || match(tmp->host, host) == 0)) {
			if (!attach_conf(cptr, tmp) && !first)
				first = tmp;
		}
		else if ((tmp->status & statmask) && !IsIllegal(tmp) && (tmp->status & CONF_SERVER_MASK) && (tmp->host && mycmp(tmp->host, host) == 0)) {
			if (!attach_conf(cptr, tmp) && !first)
			first = tmp;
		}
	}
	return (first);
}


/* find a conf entry which matches the hostname and has the same name. */
struct ConfItem *find_conf_exact(char *name, char *user, char *host, int statmask)
{
	struct ConfItem *tmp;
	char userhost[USERLEN+HOSTLEN+3];

	sprintf(userhost, "%s@%s", user, host);

	for (tmp = conf; tmp; tmp = tmp->next) {
		if (!(tmp->status & statmask) || !tmp->name || !tmp->host || mycmp(tmp->name, name))
			continue;
		/* Accept if the *real* hostname (usually sockecthost)
		 * socket host) matches *either* host or name field
		 * of the configuration.
		 */
		if (ip_match(tmp->host,userhost) && match(tmp->host,userhost))
			continue;
		if (tmp->status & (CONF_OPERATOR|CONF_LOCOP)) {
			if (tmp->clients < MaxLinks(Class(tmp)))
				return tmp;
			else
				continue;
		}
		else
			return tmp;
	}
	return NULL;
}


struct ConfItem *find_conf_name(char *name, int statmask)
{
	struct ConfItem *tmp;

	for (tmp = conf; tmp; tmp = tmp->next) {
		/* Accept if the *real* hostname (usually sockecthost)
		 * matches *either* host or name field of the configuration.
		 */
		if ((tmp->status & statmask) && (!tmp->name || match(tmp->name, name) == 0))
			return tmp;
	}
	return NULL;
}


struct ConfItem *find_conf_servern(char *name)
{
	struct ConfItem *tmp;

	for (tmp = conf; tmp; tmp = tmp->next) {
		/* Accept if the *real* hostname (usually sockecthost)
		 * matches *either* host or name field of the configuration.
		 */
		if ((tmp->status & CONF_NOCONNECT_SERVER) && (!tmp->name || match(tmp->name, name) == 0))
			return tmp;
	}
	return NULL;
}


struct ConfItem *find_conf(Link *lp, char *name, int statmask)
{
	struct ConfItem *tmp;
	int namelen = name ? strlen(name) : 0;

	if (namelen > HOSTLEN)
		return (struct ConfItem *) 0;

	for (; lp; lp = lp->next) {
		tmp = lp->value.aconf;

		if ((tmp->status & statmask) &&
			(((tmp->status & (CONF_SERVER_MASK|CONF_HUB)) &&
			tmp->name && !mycmp(tmp->name, name)) ||
			((tmp->status & (CONF_SERVER_MASK|CONF_HUB)) == 0 &&
			tmp->name && !match(tmp->name, name))))
				return tmp;
	}
	return NULL;
}


/* V-Line functions --  Borg88 */

struct ConfItem *find_vhost(char *ip)
{
	struct ConfItem *tmp;
	for(tmp = conf; tmp; tmp = tmp->next) {
		if((tmp->status & CONF_VHOST) && (!ip_match(tmp->host, ip) || !match(tmp->host, ip)))
			return tmp;
	}
	return(0);
}


void make_vhost(struct ConfItem *tmp, char *ip, char *dptr)
{
	char str[32], *state;

	if (tmp->port == 1) {
		if (*tmp->passwd)
			strcpy(dptr, tmp->passwd);
		else
			strcpy(dptr, "host-");

		strcpy(str, ip);
		strtoken(&state, str, ".");
		strtoken(&state, 0, ".");
		strtoken(&state, 0, ".");
		strcat(dptr, strtoken(&state, 0, "."));
		strcat(dptr, ".");
		strcat(dptr, tmp->name);
		return;
	}
	else if (tmp->port == 2) {
		int shift = atoi(strchr(tmp->host, '/') + 1);
		u_int32_t mask = shift ? ~(0xffffffff << (32 - shift))
			: 0xffffffff;
		u_int32_t addr = ntohl(inet_addr(ip)) & mask;

		str[0] = '\0';

		/* make sure that we don't mess around /32 */
		if (mask) {
			/* roll the mask left to the first non-empty octet */
			for (; !(mask & 0xff000000); mask <<= 8, addr <<= 8);

			/* print the remaining octets */
			do {
				sprintf(str + strlen(str), "%d.",
					(addr & 0xff000000) >> 24);

				mask <<= 8;
				addr <<= 8;
			} while (mask);
		} else {
			str[0] = '.';
			str[1] = '\0';
		}

		strcpy(dptr, *tmp->passwd ? tmp->passwd : "host-");
		strcat(dptr, str);
		strcat(dptr, tmp->name);
		return;
	}

	strcpy(dptr, tmp->name);
}


/* Added for new access check -- meLazy */
struct ConfItem *find_conf_host(Link *lp, char *host, int statmask)
{
	struct ConfItem *tmp;
	int hostlen = host ? strlen(host) : 0;

	if (hostlen > HOSTLEN || BadPtr(host))
		return (struct ConfItem *)NULL;

	for (; lp; lp = lp->next) {
		tmp = lp->value.aconf;
		if (tmp->status & statmask && (!(tmp->status & CONF_SERVER_MASK || tmp->host) || (tmp->host && (!ip_match(tmp->host,host) || !match(tmp->host,host)))))
			return tmp;
	}
	return NULL;
}

/* find_conf_entry
 *
 * - looks for a match on all given fields.
 */
struct ConfItem *find_conf_entry(struct ConfItem *aconf, u_int mask)
{
	struct ConfItem *bconf;

	for (bconf = conf, mask &= ~CONF_ILLEGAL; bconf; bconf = bconf->next) {
		if (!(bconf->status & mask) || (bconf->port != aconf->port))
			continue;

		if ((BadPtr(bconf->host) && !BadPtr(aconf->host)) ||
			(BadPtr(aconf->host) && !BadPtr(bconf->host)))
				continue;

		if (!BadPtr(bconf->host) && mycmp(bconf->host, aconf->host))
				continue;

		if ((BadPtr(bconf->passwd) && !BadPtr(aconf->passwd)) ||
			(BadPtr(aconf->passwd) && !BadPtr(bconf->passwd)))
				continue;

		if (!BadPtr(bconf->passwd) &&
			mycmp(bconf->passwd, aconf->passwd))
				continue;

		if ((BadPtr(bconf->name) && !BadPtr(aconf->name)) ||
			(BadPtr(aconf->name) && !BadPtr(bconf->name)))
				continue;

		if (!BadPtr(bconf->name) && mycmp(bconf->name, aconf->name))
				continue;
		break;
	}
	return bconf;
}


/* find_temp_conf_entry
 *
 * - looks for a match on all given fields for a TEMP conf line.
 *  Right now the passwd,port, and class fields are ignored, because it's
 *  only useful for k:lines anyway.  -Russell   11/22/95
 *  1/21/95 Now looks for any conf line.  I'm leaving this routine and its
 *  call in because this routine has potential in future upgrades. -Russell
 */
struct ConfItem *find_temp_conf_entry(struct ConfItem *aconf, u_int mask)
{
	struct ConfItem *bconf;

	for (bconf = conf, mask &= ~CONF_ILLEGAL; bconf; bconf = bconf->next) {
		/* kline/unkline/kline fix -- Barubary */
		if (bconf->status & CONF_ILLEGAL)
			continue;

		if (!(bconf->status & mask) || (bconf->port != aconf->port))
			continue;

		if ((BadPtr(bconf->host) && !BadPtr(aconf->host)) ||
			(BadPtr(aconf->host) && !BadPtr(bconf->host)))
				continue;

		if (!BadPtr(bconf->host) && mycmp(bconf->host, aconf->host))
				continue;

		if ((BadPtr(bconf->name) && !BadPtr(aconf->name)) ||
			(BadPtr(aconf->name) && !BadPtr(bconf->name)))
				continue;

		if (!BadPtr(bconf->name) && mycmp(bconf->name, aconf->name))
				continue;
		break;
	}
	return bconf;
}


aSqlineItem *find_sqline_nick(char *nickmask)
{
	aSqlineItem *asqline;

	for (asqline = sqline; asqline; asqline = asqline->next) {
		if(!BadPtr(asqline->sqline) && (asqline->status != CONF_ILLEGAL) && !mycmp(asqline->sqline, nickmask))
			return asqline;
	}
	return NULL;
}

aSqlineItem *find_sqline_match(char *nickname)
{
	aSqlineItem *asqline;

	for (asqline = sqline; asqline; asqline = asqline->next) {
		if (!BadPtr(asqline->sqline) && (asqline->status != CONF_ILLEGAL) && !match(asqline->sqline,nickname))
			return asqline;
	}
	return NULL;
}


/*
**      parv[0] = sender prefix
**      parv[1] = server
**      parv[2] = +/-
**
*/
int m_svsnoop(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	struct ConfItem *aconf;

	if (!(check_registered(sptr) && IsULine(cptr, sptr) && parc > 2))
		return 0;

	if (!hunt_server(cptr, sptr, ":%s SVSNOOP %s :%s", 1, parc, parv) != HUNTED_ISME) {
		if (parv[2][0] == '+')
			for (aconf = conf; conf; aconf = aconf->next) {
				if (aconf->status & CONF_OPERATOR || aconf->status & CONF_LOCOP)
					aconf->status = CONF_ILLEGAL;
			}
			else
				rehash(&me, &me, 2);
	}
	return 0;
}


/* rehash
 *
 * Actual REHASH service routine. Called with sig == 0 if it has been called
 * as a result of an operator issuing this command, else assume it has been
 * called as a result of the server receiving a HUP signal.
 */
int rehash(aClient *cptr, aClient *sptr, int sig)
{
	int i, ret = 0;
	aClient *acptr;
	struct ConfItem **tmp = &conf, *tmp2;
	aClass *cltmp;

	oper_closed = 0;

	if (sig == 1)
		sendto_ops("Received SIGHUP, reloading configuration.");

	for (i = 0; i <= highest_fd; i++)
		if ((acptr = local[i]) && !IsMe(acptr)) {
			/* Nullify any references from client structures to
			 * this host structure which is about to be freed.
			 * Could always keep reference counts instead of
			 * this....-avalon
			 */
			acptr->hostp = NULL;

#if defined(R_LINES_REHASH) && !defined(R_LINES_OFTEN)
			if (find_restrict(acptr)) {
				sendto_ops("Restricting %s, closing lp", get_client_name(acptr,FALSE));
			if (exit_client(cptr,acptr,sptr,"R-lined") == FLUSH_BUFFER)
				ret = FLUSH_BUFFER;
		}
#endif
	}

	while ((tmp2 = *tmp))
		if (tmp2->clients || tmp2->status & CONF_LISTEN_PORT) {
			/* Configuration entry is still in use by some
			 * local clients, cannot delete it--mark it so
			 * that it will be deleted when the last client
			 * exits...
			 */
			if (!(tmp2->status & (CONF_LISTEN_PORT|CONF_CLIENT))) {
				*tmp = tmp2->next;
				tmp2->next = NULL;
			}
			else
				tmp = &tmp2->next;
			tmp2->status |= CONF_ILLEGAL;
		}
		else {
			*tmp = tmp2->next;
			/* free expression trees of connect rules */
			if ((tmp2->status & (CONF_CRULEALL|CONF_CRULEAUTO)) && (tmp2->passwd != NULL))
				crule_free (&(tmp2->passwd));
			free_conf(tmp2);
		}

		/* We don't delete the class table, rather mark all entries
		 * for deletion. The table is cleaned up by check_class. - avalon
		 */
		for(cltmp = NextClass(FirstClass()); cltmp; cltmp = NextClass(cltmp))
			MaxLinks(cltmp) = -1;

		if (sig != 2)
			flush_cache();

		initconf(0);
		close_listeners();

		/* flush out deleted I and P lines although still in use. */
		for (tmp = &conf; (tmp2 = *tmp); )
			if (!(tmp2->status & CONF_ILLEGAL))
				tmp = &tmp2->next;
			else {
				*tmp = tmp2->next;
				tmp2->next = NULL;
				if (!tmp2->clients)
					free_conf(tmp2);
			}

		/* Added to make sure K-lines are checked -- Barubary */
		check_pings(time(NULL), 1);

		/* Recheck all U-lines -- Barubary */
		for (i = 0; i < highest_fd; i++)
			if ((acptr = local[i]) && !IsMe(acptr)) {
				if (find_conf_host(acptr->from->confs, acptr->name, CONF_UWORLD) || (acptr->user && find_conf_host(acptr->from->confs, acptr->srvptr->name, CONF_UWORLD)))
					acptr->flags |= FLAGS_ULINE;
				else
					acptr->flags &= ~FLAGS_ULINE;
			}

	reset_help();

	return ret;
}


static char *get_field(char **line)
{
	int escaped = 0;
	char *start = *line, *p, *dst;

	for (p = start; (p = strchr(p, ':')) != NULL; ) {
		/* skip escaped colons */
		if (p != start && *(p - 1) == '\\') {
			escaped = 1;
			++p;
			continue;
		}
		/* an unescaped colon was found */
		*p++ = '\0';
		*line = p;
		break;
	}

	if (!p)
		*line += strlen(start);

	if (escaped) {
		for (dst = start, p = start; *p; ) {
			if (p[0] == '\\' && p[1] == ':')
				++p;
			*dst++ = *p++;
		}
		*dst = '\0';
	}

	return start;
}


static const int conf_lines[] = {
	'A',	CONF_ADMIN,
	'a',	CONF_SADMIN,
	'C',	CONF_CONNECT_SERVER,
	'c',	CONF_CONNECT_SERVER,
	'D',	CONF_CRULEALL,
	'd',	CONF_CRULEAUTO,
	'G',	CONF_CONFIG,
	'g',	CONF_CONFIG,
	'H',	CONF_HUB,
	'h',	CONF_HUB,
	'I',	CONF_CLIENT,
	'i',	CONF_CLIENT,
	'K',	CONF_KILL,
	'k',	CONF_KILL,
	'L',	CONF_LEAF,
	'l',	CONF_LEAF,
	'M',	CONF_ME,
	'm',	CONF_ME,
	'N',	CONF_NOCONNECT_SERVER,
	'n',	CONF_NOCONNECT_SERVER,
	'O',	CONF_OPERATOR,
	'o',	CONF_OPERATOR,
	'P',	CONF_LISTEN_PORT,
	'p',	CONF_LISTEN_PORT,
	'Q',	CONF_QUARANTINED_NICK,
	'q',	CONF_QUARANTINED_SERVER,
#ifdef R_LINES
	'R',	CONF_RESTRICT,
	'r',	CONF_RESTRICT,
#endif
	'S',	CONF_SERVICE,
	's',	CONF_SERVICE,
	'U',	CONF_UWORLD,
	'u',	CONF_UWORLD,
	'V',	CONF_VHOST,
	'v',	CONF_VHOST,
	'X',	CONF_DRPASS,
	'x',	CONF_DRPASS,
	'Y',	CONF_CLASS,
	'y',	CONF_CLASS,
	'Z',	CONF_ZAP,
	'z',	CONF_ZAP,
	0
};

static const int oper_access[] = {
	'O',	OFLAG_LOCOP,
	'o',	OFLAG_OPER,
	'A',	OFLAG_ADMIN,
	'a',	OFLAG_SADMIN,
	'*',	OFLAG_LOCOP|OFLAG_OPER|OFLAG_ADMIN|OFLAG_SADMIN,
	0
};

static const int def_umodes[] = {
	'i',	UMODE_INVISIBLE,
	'f',	UMODE_FLOOD,
	'x',	UMODE_HMASK,
	'n',	UMODE_UWNOTICE,
	0
};

static const int noconn_flags[] = {
	'Z',	PROTO_ZIP,
	0
};


static int flags_match(char *flags, const int *modes)
{
	int access;
	const int *mode;

	for (access = 0; *flags && *flags != ':'; ++flags) {
		for (mode = modes; *mode; mode += 2) {
			if (*mode == (int)*flags) {
				access |= *(mode + 1);
				break;
			}
		}
	}

	return access;
}


static void parse_conf_line_old(char *line, int opt)
{
	char *p, *state;
	aConfItem *aconf = make_conf();

	if (!(aconf->status = flags_match(p = get_field(&line), conf_lines))) {
		Debug((DEBUG_ERROR, "Error in config file: %s:%s", p, line));
		goto clean;
	}

	if (IsIllegal(aconf))
		goto clean;

	/* parse fields */
	if ((p = get_field(&line)) != NULL)
		aconf->host = strdup(p);
	if ((p = get_field(&line)) != NULL)
		aconf->passwd = strdup(p);
	if ((p = get_field(&line)) != NULL)
		aconf->name = strdup(p);

	p = get_field(&line);

	if (aconf->status & CONF_OPS) {
		aconf->port = flags_match(p, oper_access);
		/* default to locop (if no flags listed) */
		if (!(aconf->port & OFLAG_OPER))
			aconf->status = CONF_LOCOP;
	}
	else if (aconf->status & CONF_NOCONNECT_SERVER) {
		aconf->port = flags_match(p, noconn_flags);
	}
	else aconf->port = atoi(p);

	Class(aconf) = find_class(atoi(get_field(&line)));

	/* "general config" - misc settings, deprecated by .._new() */
	if ((aconf->status & CONF_CONFIG) == CONF_CONFIG) {
#ifdef USE_CASETABLES
		if (mycmp(aconf->host, "casetable") == 0) {
			if (atoi(aconf->passwd) == 1) {
				casetable = 1;
				touppertab = touppertab2;
				tolowertab = tolowertab2;
			}
			else if (casetable == 1)
				sendto_ops("WARNING -- rehash tried to reduce "
					"casetable value, which is illegal.  "
					"Restart to reduce it.");
		}
#endif
		goto clean;
	}

	/* TODO: needs massive rewriting, weird code. */
	if (aconf->status == CONF_VHOST) {
		char str[32], *p;
		u_long bf;
		int t, t1;

		if (BadPtr(aconf->name) || check_hostname(aconf->name))
			goto clean;

		/* TODO: find out why it's 20 */
		if (strlen(aconf->host) > 20)
			goto clean;

		aconf->port = 0;

		if ((p = strchr(aconf->host, '/')) != NULL) {
			if ((bf = atoi(p + 1)) > 32)
				goto clean;

			aconf->port = 2;
			strlcpy(str, aconf->host, p - aconf->host + 1);
		}
		else strlcpy(str, aconf->host, sizeof(str));

		if (!(p = strtoken(&state, str, ".")) || !isstrdigit(p))
			goto clean;
		if (!(p = strtoken(&state, 0, ".")) || !isstrdigit(p))
			goto clean;
		if (!(p = strtoken(&state, 0, ".")) || !isstrdigit(p))
			goto clean;
		if (!(p = strtoken(&state, 0, ".")))
			goto clean;

		if (aconf->port != 2 && !strcmp(p, "*"))
			aconf->port = 1;
		else if (!isstrdigit(p))
			goto clean;

		if ((p = strtoken(&state, 0, ".")) != NULL)
			goto clean;

		if (!aconf->port) {
			if (strlen(aconf->name) > HOSTLEN)
				goto clean;
			goto attach;
		}

		if (!(t1 = strlen(aconf->passwd)))
			t1 = 2;

		if ((t = t1 + strlen(aconf->name) + 10) > HOSTLEN)
			goto clean;
	}

	if ((aconf->status == CONF_ZAP) && (check_ipmask(aconf->host)))
		goto clean;

	if (aconf->status & CONF_CLASS) {
		add_class(atoi(aconf->host), atoi(aconf->passwd),
			atoi(aconf->name), aconf->port, atoi(get_field(&line)));
		goto clean;
	}

	if (aconf->status & (CONF_CLIENT_MASK|CONF_LISTEN_PORT)) {
		if (Class(aconf) == 0)
			Class(aconf) = find_class(0);
		if (MaxLinks(Class(aconf)) < 0)
			Class(aconf) = find_class(0);
	}

	/* we no longer pre-load the charsets, we do that by request.
	if ((aconf->status & CONF_LISTEN_PORT) && !BadPtr(aconf->name))
		codepage_load(aconf->name);
	*/

	if (aconf->status & (CONF_LISTEN_PORT|CONF_CLIENT)) {
		aConfItem *bconf;

		if ((bconf = find_conf_entry(aconf, aconf->status)) != NULL) {
			delist_conf(bconf);
			bconf->status &= ~CONF_ILLEGAL;
			if (aconf->status == CONF_CLIENT) {
				bconf->class->links -= bconf->clients;
				bconf->class = aconf->class;
				if (bconf->class)
					bconf->class->links += bconf->clients;
			}
			free_conf(aconf);
			aconf = bconf;
		}
		else if (aconf->host && (aconf->status == CONF_LISTEN_PORT)
		&& aconf->port >= 1024)
			add_listener(aconf);
	}

	if (aconf->status & CONF_SERVER_MASK)
		if (BadPtr(aconf->host) || BadPtr(aconf->name))
			goto clean;

	if (aconf->status & (CONF_SERVER_MASK|CONF_LOCOP|CONF_OPERATOR)) {
		if (!strchr(aconf->host, '@') && (*aconf->host != '/')) {
			p = malloc(strlen(aconf->host) + 3);
			sprintf(p, "*@%s", aconf->host);
			free(aconf->host);
			aconf->host = p;
		}
	}

	if (aconf->status & CONF_SERVER_MASK) {
		if (BadPtr(aconf->passwd))
			goto clean;
		else if (!(opt & BOOT_QUICK))
			lookup_confhost(aconf);
	}

	if (aconf->status & (CONF_CRULEALL|CONF_CRULEAUTO)) {
		free(aconf->passwd);
		if (!(aconf->passwd = crule_parse(aconf->name)))
			goto clean;
	}

	if (aconf->status == CONF_ME) {
		strlcpy(me.info, aconf->name, sizeof(me.info));
		if (!me.name[0] && !BadPtr(aconf->host))
			strlcpy(me.name, aconf->host, sizeof(me.name));
		if (!BadPtr(aconf->passwd) && strcmp(aconf->passwd, "*"))
			me.ip.s_addr = inet_addr(aconf->passwd);
		else
			me.ip.s_addr = INADDR_ANY;
		if (portnum < 0 && (aconf->port >= 0))
			portnum = aconf->port;
	}

	if (aconf->status == CONF_KILL)
		aconf->tmpconf = KLINE_PERM;

attach:
	collapse(aconf->host);
	collapse(aconf->name);
	Debug((DEBUG_NOTICE, "Read Init: (%d) (%s) (%s) (%s) (%d) (%d)",
		aconf->status, aconf->host, aconf->passwd, aconf->name,
		aconf->port, Class(aconf)));

	aconf->next = conf;
	conf = aconf;
	return;

clean:
	free_conf(aconf);
	return;
}

static void get_word(char *value, char **target)
{
	size_t size;
	if (*target) {
		free(*target);
		*target = NULL;
	}
	if (!BadPtr(value) && (size = strcspn(value, " \t;")))
		value[size] = '\0';
	if (!BadPtr(value))
		*target = strdup(value);
}

static void get_line(char *value, char **target)
{
	if (*target) {
		free(*target);
		*target = NULL;
	}
	if (!BadPtr(value))
		*target = strdup(value);
}


/* TODO: add some strict control over values such as ServerMask */
static void parse_conf_line_new(char *para, char *value)
{
	if (!strcasecmp(para, "NetworkName"))
		get_word(value, &network_name);
	else if (!strcasecmp(para, "KLineAddress"))
		get_word(value, &kline_address);
	else if (!strcasecmp(para, "ContactURL"))
		get_word(value, &contact_url);
	else if (!strcasecmp(para, "ServicesHost")) {
		get_word(value, &services_host);
		if (check_hostname(services_host)) {
			Debug((DEBUG_ERROR, "Errorneous services host name: %s",
				services_host));
			free(services_host);
			services_host = NULL;
		}
	}
	else if (!strcasecmp(para, "ServerMask"))
		get_word(value, &server_mask);
	else if (!strcasecmp(para, "DefaultUserMode"))
		def_umode = flags_match(value, def_umodes);
	else if (!strcasecmp(para, "NickServ"))
		get_word(value, &ns_nick);
	else if (!strcasecmp(para, "ChanServ"))
		get_word(value, &cs_nick);
	else if (!strcasecmp(para, "MemoServ"))
		get_word(value, &ms_nick);
	else if (!strcasecmp(para, "OperServ"))
		get_word(value, &os_nick);
	else if (!strcasecmp(para, "NickChangeDelay"))
		nickchange_delay = atoi(value);
	else if (!strcasecmp(para, "ShowConnectInfo"))
		show_connect_info = 1;
	else if (!strcasecmp(para, "ForceMOTD"))
		force_motd = 1;
	/* deprecated */
	else if (!strcasecmp(para, "CodepagePath"))
		get_line(value, &codepage_path);
	else if (!strcasecmp(para, "CharSetPath"))
		get_line(value, &codepage_path);
	else if (!strcasecmp(para, "OpersLog"))
		get_line(value, &opers_log);
	else if (!strcasecmp(para, "ClientsLog"))
		get_line(value, &clients_log);
}

#define kill_string(str) if (str) { free(str); str = NULL; }

/* read configuration file */
int initconf(int opt)
{
	FILE *in;
	size_t size;
	char linebuf[1024], *p, c;

	if (!(in = fopen(configfile, "r")))
		return -1;

	/* reset previously configured settings */
	kill_string(network_name)
	kill_string(kline_address)
	kill_string(contact_url)
	kill_string(services_host)
	kill_string(server_mask)
	kill_string(codepage_path)
	def_umode = 0;
	nickchange_delay = 0;
	show_connect_info = 0;
	force_motd = 0;

	while (fgets(linebuf, sizeof(linebuf), in) != 0) {
		/* truncate the line if EOL present */
		if (linebuf[size = strcspn(linebuf, "\r\n")])
			linebuf[size] = '\0';
		/* EOL not found - skip the rest of the line.  this is the
		 * slowest method but we're not building a high-performance
		 * file reading routine, we're just reading a [small] config.
		 */
		else for (; !feof(in) && (fgets(&c, 1, in) != 0)
			&& !(c == '\r' || c == '\n'); );

		if (!*linebuf || *linebuf == '#')
			continue;

		/* old-style configuration lines */
		if (linebuf[1] == ':')
			parse_conf_line_old(linebuf, opt);

		/* split to parameter and value */
		else {
			for (p = linebuf; *p && !(*p == ' ' || *p == '\t'); ++p);
			for (; *p == ' ' || *p == '\t'; *p++ = '\0');
			parse_conf_line_new(linebuf, p);
		}
	}

	fclose(in);

	/* apply some defaults */
	if (services_host) {
		if (BadPtr(ns_nick))
			ns_nick = strdup("NickServ");
		if (BadPtr(cs_nick))
			cs_nick = strdup("ChanServ");
		if (BadPtr(ms_nick))
			ms_nick = strdup("MemoServ");
		if (BadPtr(os_nick))
			os_nick = strdup("OperServ");
	}

	if (!codepage_path)
		codepage_path = strdup("charsets");

	check_class();
	nextping = nextconnect = time(NULL);
	return 0;
}


/* lookup_confhost
 *   Do (start) DNS lookups of all hostnames in the conf line and convert
 * an IP addresses in a.b.c.d number for to IP#s.
 */
static int lookup_confhost(struct ConfItem *aconf)
{
	char *s;

	if (BadPtr(aconf->host) || BadPtr(aconf->name))
		goto badlookup;
	if ((s = strchr(aconf->host, '@')))
		s++;
	else
		s = aconf->host;

	if(aconf->status&CONF_NOCONNECT_SERVER) {
		aconf->ipnum.s_addr=-1;
		return(0);
	}

	if (!isalpha(*s) && !isdigit(*s))
		goto badlookup;

	/* Prepare structure in case we have to wait for a
	 * reply which we get later and store away.
	 */
	if (isdigit(*s))
		aconf->ipnum.s_addr = inet_addr(s);

	if ((int)aconf->ipnum.s_addr == -1)
		goto badlookup;

	return 0;

badlookup:
	memset(&aconf->ipnum, 0, sizeof(struct in_addr));
	Debug((DEBUG_ERROR,"Host/server name error: (%s) (%s)", aconf->host, aconf->name));
	return -1;
}


int find_kill(aClient *cptr)
{
	char reply[256], *host, *vhost, *name;
	struct ConfItem *tmp;

	if (!cptr->user)
		return 0;

	host = cptr->sockhost;
	vhost = cptr->vhost;
	name = cptr->user->username;

	if (strlen(host)  > (size_t) HOSTLEN || (name ? strlen(name) : 0) > (size_t) HOSTLEN)
		return (0);

	reply[0] = '\0';

	for (tmp = conf; tmp; tmp = tmp->next) {
		if ((tmp->status == CONF_KILL) && tmp->host && tmp->name &&
			(!ip_match(tmp->host,host) || !match(tmp->host,host) || !match(tmp->host,vhost)) &&
			(!name || match(tmp->name, name) == 0) &&
			(!tmp->port || (tmp->port == cptr->acpt->port))) {
				/* can short-circuit evaluation - not taking chances
				 * cos check_time_interval destroys tmp->passwd - Mmmm
				 */
				if (BadPtr(tmp->passwd))
					break;
				else if (is_comment(tmp->passwd))
					break;
				else if (check_time_interval(tmp->passwd, reply))
					break;
		}
	}

		if (tmp) {
			if(tmp->passwd && is_comment(tmp->passwd)) {
				strcpy(gcomment, "K-Lined: ");
				strncpyzt(gcomment+9, tmp->passwd, 256-9);
			}
			else strcpy(gcomment, "K-Lined");
		}

	   return (tmp ? -1 : 0);
}

char find_zap(aClient *cptr)
{
	struct ConfItem *tmp;

	for(tmp=conf; tmp; tmp=tmp->next) {
		if (tmp->status == CONF_ZAP && tmp->host && (!ip_match(tmp->host, inetntoa((char *)&cptr->ip)) || !match(tmp->host, inetntoa((char *)&cptr->ip)))) {
			strcpy(gcomment,"Z-Lined");
			return 1;
		}
	}
	return 0;
}


int find_kill_byname(char *host, char *name)
{
	struct ConfItem *tmp;

	for (tmp = conf; tmp; tmp = tmp->next) {
		if ((tmp->status == CONF_KILL) && tmp->host && tmp->name &&
			(match(tmp->host, host) == 0) &&
			(!name || match(tmp->name, name) == 0))
				return 1;
	}

	return 0;
}


#ifdef R_LINES
/* find_restrict works against host/name and calls an outside program
 * to determine whether a client is allowed to connect.  This allows
 * more freedom to determine who is legal and who isn't, for example
 * machine load considerations.  The outside program is expected to
 * return a reply line where the first word is either 'Y' or 'N' meaning
 * "Yes Let them in" or "No don't let them in."  If the first word
 * begins with neither 'Y' or 'N' the default is to let the person on.
 * It returns a value of 0 if the user is to be let through -Hoppie
 */
int find_restrict(aClient *cptr)
{
	struct ConfItem *tmp;
	char reply[80], temprpl[80], *rplhold = reply, *host, *name, *s, rplchar = 'Y';
	int pi[2], rc = 0, n;

	if (!cptr->user)
		return 0;

	name = cptr->user->username;
	host = cptr->sockhost;
	Debug((DEBUG_INFO, "R-line check for %s[%s]", name, host));

	for (tmp = conf; tmp; tmp = tmp->next) {
		if (tmp->status != CONF_RESTRICT ||
			(tmp->host && host && match(tmp->host, host)) ||
			(tmp->name && name && match(tmp->name, name)))
				continue;

		if (BadPtr(tmp->passwd)) {
			sendto_ops("Program missing on R-line %s/%s, ignoring", name, host);
			continue;
		}

		if (pipe(pi) == -1) {
			report_error("Error creating pipe for R-line %s:%s", &me);
			return 0;
		}

		switch (rc = fork()) {
			case -1 :
				report_error("Error forking for R-line %s:%s", &me);
				return 0;

			case 0: {
				int i;

				close(pi[0]);
				for (i = 2; i < MAXCONNECTIONS; i++)
					if (i != pi[1])
						close(i);
				if (pi[1] != 2)
					dup2(pi[1], 2);
				dup2(2, 1);
				if (pi[1] != 2 && pi[1] != 1)
					close(pi[1]);
				execlp(tmp->passwd, tmp->passwd, name, host, 0);
				exit(-1);
			}
			default:
				close(pi[1]);
				break;
		}

		*reply = '\0';
		dgets(-1, NULL, 0); /* make sure buffer marked empty */
		while ((n = dgets(pi[0], temprpl, sizeof(temprpl)-1)) > 0) {
			temprpl[n] = '\0';

			if ((s = strchr(temprpl, '\n')))
				*s = '\0';

			if (strlen(temprpl) + strlen(reply) < sizeof(reply)-2)
				sprintf(rplhold, "%s %s", rplhold, temprpl);
			else {
				sendto_ops("R-line %s/%s: reply too long!", name, host);
				break;
			}
		}

		dgets(-1, NULL, 0); /* make sure buffer marked empty */
		close(pi[0]);
		kill(rc, SIGKILL); /* cleanup time */
		wait(0);

		rc = 0;
		while (*rplhold == ' ')
			rplhold++;
		rplchar = *rplhold; /* Pull out the yes or no */
		while (*rplhold != ' ')
			rplhold++;
		while (*rplhold == ' ')
			rplhold++;
		strcpy(reply,rplhold);
		rplhold = reply;

		if ((rc = (rplchar == 'n' || rplchar == 'N')))
			break;
	}
	if (rc) {
		sendto_one(cptr, ":%s %d %s :Restriction: %s", me.name, ERR_YOUREBANNEDCREEP, cptr->name, reply);
		return -1;
	}
	return 0;
}
#endif

/*  output the reason for being k lined from a file  - Mmmm
 * sptr is server
 * parv is the sender prefix
 * filename is the file that is to be output to the K lined client
 */
int m_killcomment(aClient *sptr, char *parv, char *filename)
{
	int fd;
	char line[80], *tmp;
	struct stat sb;
	struct tm *tm;

	/* stop NFS hangs...most systems should be able to open a file in
	 * 3 seconds. -avalon (curtesy of wumpus)
	 */
	fd = open(filename, O_RDONLY);
	if (fd == -1) {
		sendto_one(sptr, err_str(ERR_NOMOTD), me.name, parv);
		sendto_one(sptr, ":%s %d %s :*** You are not welcome to this server.",
			me.name, ERR_YOUREBANNEDCREEP, parv);
		return 0;
	}

	fstat(fd, &sb);
	tm = localtime(&sb.st_mtime);
	dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
	while (dgets(fd, line, sizeof(line)-1) > 0) {
		if ((tmp = strchr(line,'\n')) != NULL)
			*tmp = '\0';
		if ((tmp = strchr(line,'\r')) != NULL)
			*tmp = '\0';
		sendto_one(sptr, rpl_str(RPL_MOTD), me.name, parv, line);
	}
	sendto_one(sptr, ":%s %d %s :*** You are not welcome to this server.",
		me.name, ERR_YOUREBANNEDCREEP, parv);
	dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
	close(fd);
	  return 0;
}


/* is the K line field an interval or a comment? - Mmmm */

static int is_comment(char *comment)
{
	size_t i;
	for (i = 0; i < strlen(comment); i++)
		if ( (comment[i] != ' ') && (comment[i] != '-') && (comment[i] != ',') && ((comment[i] < '0') || (comment[i] > '9')))
			return(1);

	return(0);
}


/* check against a set of time intervals */

static int check_time_interval(char *interval, char *reply)
{
	struct tm *tptr;
	time_t  tick;
	char *p;
	int perm_min_hours, perm_min_minutes, perm_max_hours, perm_max_minutes;
	int now, perm_min, perm_max;

	tick = time(NULL);
	tptr = localtime(&tick);
	now = (tptr->tm_hour) * 60 + tptr->tm_min;

	while (interval) {
		p = strchr(interval, ',');

		if (p)
			*p = '\0';

		if (sscanf(interval, "%2d%2d-%2d%2d", &perm_min_hours, &perm_min_minutes, &perm_max_hours, &perm_max_minutes) != 4) {
			if (p)
				*p = ',';
			return 0;
		}

		if (p)
			*(p++) = ',';

		perm_min = 60 * perm_min_hours + perm_min_minutes;
		perm_max = 60 * perm_max_hours + perm_max_minutes;

		/* The following check allows intervals over midnight ... */
		if ((perm_min < perm_max) ? (perm_min <= now && now <= perm_max) : (perm_min <= now || now <= perm_max)) {
			sprintf(reply, ":%%s %%d %%s :%s %d:%02d to %d:%02d.", "You are not allowed to connect from", perm_min_hours, perm_min_minutes, perm_max_hours, perm_max_minutes);
			return(ERR_YOUREBANNEDCREEP);
		}

		if ((perm_min < perm_max) ? (perm_min <= now + 5 && now + 5 <= perm_max) : (perm_min <= now + 5 || now + 5 <= perm_max)) {
			sprintf(reply, ":%%s %%d %%s :%d minute%s%s", perm_min-now,(perm_min-now)>1?"s ":" ", "and you will be denied for further access");
			return ERR_YOUWILLBEBANNED;
		}
		interval = p;
	}
	return(0);
}


/*    m_sqline
 *     parv[0] = sender
 *     parv[1] = nickmask
 *     parv[2] = reason
 */
int m_sqline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aSqlineItem *asqline;

	if (!IsServer(sptr) || parc < 2)
		return 0;

	if(parv[2])
		sendto_serv_butone(cptr, ":%s SQLINE %s :%s", parv[0], parv[1], parv[2]);
	else
		sendto_serv_butone(cptr, ":%s SQLINE %s", parv[0], parv[1]);

	asqline = make_sqline();

	if (parv[2])
		asqline->reason = strdup(parv[2]);

	if (parv[1])
		asqline->sqline = strdup(parv[1]);

	if (!find_sqline_nick(parv[1])) {
		asqline->next = sqline;
		sqline = asqline;
		asqline = NULL;
	}

	if (asqline)
		free_sqline(asqline);

	return 0;
}

/* m_unsqline
 *  parv[0] = sender
 *  parv[1] = nickmask
 */
int m_unsqline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aSqlineItem *asqline;

	if (!IsServer(sptr) || parc < 1)
		return 0;

	sendto_serv_butone(cptr, ":%s UNSQLINE %s", parv[0], parv[1]);

	if(!(asqline = find_sqline_nick(parv[1])))
		return 0;

	asqline->status = CONF_ILLEGAL;
	return 0;
}


/* m_kline;
 *     parv[0] = sender prefix
 *     parv[1] = hostname
 *     parv[2] = comment or filename
 */
int m_kline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *hosttemp, uhost[128], name[128];
	aClient *acptr;

	if (check_registered(sptr))
		return 0;

	if (!MyClient(sptr) || !IsAnOper(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc < 2) {
		send_syntax(sptr, MSG_KLINE, "KLINE nick|[user@]host [:reason ...]");
		return 0;
	}

	/* nick based kline */
	if ((acptr = find_person(parv[1], NULL))) {
		strcpy(name, acptr->user->username);
		strcpy(uhost, acptr->user->host);
	}

	else {
		if (strlen(parv[1]) > 127) {
			sendto_one(sptr,"NOTICE %s :Bad parameters", parv[0]);
			return 0;
		}

		memset(name, 0, sizeof(name));

		if ((hosttemp = strchr(parv[1], '@')) != NULL) {
			hosttemp++;
			strncpy(name, parv[1], hosttemp -1 - parv[1]);
		}
		else {
			hosttemp = parv[1];
			strcpy(name, "*");
		}

		strcpy(uhost, hosttemp);

		if (name[0] == '\0' || uhost[0] == '\0') {
			Debug((DEBUG_INFO, "KLINE: Bad field!"));
			sendto_one (sptr, "NOTICE %s :If you're going to add a userhost, at LEAST specify both fields", parv[0]);
			return 0;
		}

		if (!strcmp(uhost, "*")) {
			sendto_one (sptr, "NOTICE %s :What a sweeping K-Line.  If only your admin knew you tried that..", parv[0]);
			return 0;
		}
	}

	if (parv[2] && strlen(parv[2]))
		sendto_ops("%s added a temp K-Line %s@%s (%s)", parv[0], name, uhost, parv[2]);
	else
		sendto_ops("%s added a temp K-Line %s@%s", parv[0], name, uhost);

	add_temp_conf(CONF_KILL, uhost, parv[2], name, 0, 0, 1);
	check_pings(time(NULL), 1);
	return 0;
}


/* Rewrited. Added remote unkline. -- Borg88
 *  m_unkline
 *    parv[0] = sender prefix
 *    parv[1] = userhost
 *    parv[2] = remote server
 */
int m_unkline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int result;
	char *hosttemp = parv[1], host[128], name[128];

	if (check_registered(sptr))
		return 0;

	if (MyClient(sptr) && (!IsAnOper(sptr) || (IsLocOp(sptr) && parc > 2))) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc < 2) {
		send_syntax(sptr, MSG_UNKLINE, "UNKLINE user@host [server]");
		return 0;
	}

	if (strlen(parv[1]) > 127) {
		sendto_one(sptr,"NOTICE %s :Bad parameters (too long)", parv[0]);
		return 0;
	}

	if (parc > 2 && hunt_server(cptr, sptr, ":%s UNKLINE %s %s", 2, parc,parv) != HUNTED_ISME)
		return 0;

	if (!(hosttemp = strchr(parv[1], '@'))) {
		sendto_one(sptr,"NOTICE %s : Both user and host fields must be non-null",parv[0]);
		return 0;
	}

	memset(name, 0, sizeof(name));
	strcpy(host, ++hosttemp);
	strncpy(name, parv[1], hosttemp - 1 - parv[1]);

	if (name[0] =='\0' || host[0]=='\0') {
		sendto_one(sptr, "NOTICE %s : Both user and host fields must be non-null", parv[0]);
		return 0;
	}

	result = del_temp_conf(CONF_KILL, host, NULL, name, 0, 0, 0);

	if (result == KLINE_RET_AKILL) {
		/* akill - result = 3 */
		sendto_one(sptr, "NOTICE %s :You may not remove autokills.  Only U-Lined clients may.", parv[0]);
		return 0;
	}

	if (result == KLINE_RET_PERM) {
		/* Not a temporary line - result =2 */
		sendto_one(sptr, "NOTICE %s :You may not remove permanent K-Lines - talk to the admin", parv[0]);
		return 0;
	}

	if (result == KLINE_RET_DELOK) {
		/* Successful result = 1 */
		sendto_one(sptr, "NOTICE %s :Temp K-Line %s@%s is now removed.", parv[0], name, host);
		sendto_ops("%s removed temp K-Line %s@%s", parv[0], name, host);
		return 0;
	}

	if (result == KLINE_DEL_ERR) {
		/* Unsuccessful result = 0 */
		sendto_one(sptr, "NOTICE %s :Temporary K-Line %s@%s not found", parv[0], name, host);
		return 0;
	}

	return 0;
}


/* m_akill;
 *     parv[0] = sender prefix
 *     parv[1] = hostmask
 *     parv[2] = username
 *     parv[3] = comment
 */
int m_akill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (check_registered(sptr))
		return 0;

	if (parc < 3) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "AKILL");
		return 0;
	}

	if (IsServer(cptr) && find_conf_host(cptr->confs, sptr->name, CONF_UWORLD)) {
		if(!find_kill_byname(parv[1], parv[2])) {
#ifndef COMMENT_IS_FILE
			add_temp_conf(CONF_KILL, parv[1], parv[3], parv[2], 0, 0, 2);
#else
			add_temp_conf(CONF_KILL, parv[1], NULL, parv[2], 0, 0, 2);
#endif
		}

		if (parv[3])
			sendto_serv_butone(cptr, ":%s AKILL %s %s :%s", parv[0], parv[1], parv[2], parv[3]);
		else
			sendto_serv_butone(cptr, ":%s AKILL %s %s", parv[0], parv[1], parv[2]);
		check_pings(time(NULL), 1);
	}

	return 0;
}


/* m_rakill;
 *      parv[0] = sender prefix
 *      parv[1] = hostmask
 *      parv[2] = username
 *      parv[3] = comment
 */
int m_rakill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (check_registered(sptr))
		return 0;

	if (parc < 3) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "RAKILL");
		return 0;
	}

	if (IsServer(cptr) && find_conf_host(cptr->confs, sptr->name, CONF_UWORLD)) {
		if(find_kill_byname(parv[1], parv[2])) {
#ifndef COMMENT_IS_FILE
			del_temp_conf(CONF_KILL, parv[1], parv[3], parv[2], 0, 0, 1);
#else
			del_temp_conf(CONF_KILL, parv[1], NULL, parv[2], 0, 0, 1);
#endif
		}

		if (parv[3])
			sendto_serv_butone(cptr, ":%s RAKILL %s %s :%s", parv[0], parv[1], parv[2], parv[3]);
		else
			sendto_serv_butone(cptr, ":%s RAKILL %s %s", parv[0], parv[1], parv[2]);
		check_pings(time(NULL), 1);
	}
	return 0;
}

/* parv[0] = sender prefix, parv[1] = host, parv[2] = remote server */
int m_unzline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int result;

	if (check_registered(sptr))
		return 0;

	if (MyClient(sptr) && (!IsAnOper(sptr) || (IsLocOp(sptr) && parc > 2))) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc < 2) {
		send_syntax(sptr, MSG_UNZLINE, "UNZLINE hostmask [server]");
		return 0;
	}

	if (parc > 2 && hunt_server(cptr, sptr, ":%s UNZLINE %s %s", 2, parc, parv) != HUNTED_ISME)
		return 0;

	if (check_ipmask(parv[1])) {
		sendto_one(sptr,"NOTICE %s :Bad parameters", parv[0]);
		return 0;
	}

	result = del_temp_conf(CONF_ZAP, parv[1], 0, 0, 0, 0, 0);
	if (result == KLINE_RET_AKILL) {
		/* akill - result = 3 */
		sendto_one(sptr, "NOTICE %s :You may not remove autokills.  Only U-Lined clients may.", parv[0]);
		return 0;
	}

	if (result == KLINE_RET_PERM) {
		/* Not a temporary line - result =2 */
		sendto_one(sptr, "NOTICE %s :You may not remove permanent Z-Lines - talk to the admin", parv[0]);
		return 0;
	}

	if (result == KLINE_RET_DELOK) {
		/* Successful result = 1 */
		sendto_one(sptr, "NOTICE %s :Temp Z-Line %s is now removed.", parv[0], parv[1]);
		sendto_ops("%s removed temp Z-Line %s", parv[0], parv[1]);
		return 0;
	}

	if (result == KLINE_DEL_ERR) {
		/* Unsuccessful result = 0 */
		sendto_one(sptr, "NOTICE %s :Temporary Z-Line %s not found", parv[0], parv[1]);
		return 0;
	}

	return 0;
}


/* used in m_info() for example */
char *get_default_umode()
{
	static char tmpbuf[16];
	char *p = tmpbuf;
	const int *i = def_umodes;

	for (; *i; i += 2) {
		if (def_umode & *(i + 1))
			*p++ = (char)(*i);
	}

	*p = '\0';
	return tmpbuf;
}
